/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.jbws1869;

import org.jboss.wsf.spi.SPIProvider;
import org.jboss.wsf.spi.SPIProviderResolver;
import org.jboss.wsf.spi.deployment.Endpoint;
import org.jboss.wsf.spi.invocation.InvocationContext;
import org.jboss.wsf.spi.management.EndpointRegistry;
import org.jboss.wsf.spi.management.EndpointRegistryFactory;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.servlet.ServletException;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Properties;

/**
 * Reads a SOAP message from HTTP body and invokes an
 * Endpoint through {@link org.jboss.wsf.spi.invocation.RequestHandler#handleRequest(org.jboss.wsf.spi.deployment.Endpoint, java.io.InputStream, java.io.OutputStream, org.jboss.wsf.spi.invocation.InvocationContext)}
 */
public class DelegationServlet extends HttpServlet
{

   protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
   {
      ObjectName EP_NAME = null;
      try
      {
         EP_NAME = new ObjectName("jboss.ws:context=jaxws-jbws1869,endpoint=Endpoint");
      }
      catch (MalformedObjectNameException e) {}      

      // EndpointRegistry access
      SPIProvider spiProvider = SPIProviderResolver.getInstance().getProvider();
      EndpointRegistryFactory registryFactory = spiProvider.getSPI(EndpointRegistryFactory.class);
      EndpointRegistry reg = registryFactory.getEndpointRegistry();

      Endpoint endpoint = reg.getEndpoint(EP_NAME);
      ServletInputStream inputStream = request.getInputStream();
      OutputStream outputStream = response.getOutputStream();

      try
      {
         InvocationContext invocationContext = new InvocationContext();
         Properties props = new Properties();
         props.put("Content-Type", request.getContentType());
         invocationContext.addAttachment(Properties.class, props);
         
         endpoint.getRequestHandler().handleRequest(endpoint, inputStream, outputStream, invocationContext);
      }
      catch(Exception e)
      {
         throw new ServletException("Failed to invoke endpoint", e);        
      }
      finally
      {
         if(inputStream!=null) inputStream.close();
         if(outputStream!=null) outputStream.close();
      }

   }
}
