/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.jbws2206;

import junit.framework.Test;
import org.jboss.wsf.test.JBossWSTest;
import org.jboss.wsf.test.JBossWSTestSetup;

import javax.wsdl.Definition;
import javax.wsdl.Types;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.schema.Schema;
import javax.wsdl.extensions.schema.SchemaImport;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.xml.ws.Service;
import javax.xml.namespace.QName;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.List;
import java.util.Map;

public class JBWS2206TestCase extends JBossWSTest
{
   public static Test suite()
   {
      return new JBossWSTestSetup(JBWS2206TestCase.class, "jaxws-jbws2206.jar");
   }

   public void testClientProxy() throws Exception
   {
      URL wsdlURL = new URL("http://" + getServerHost() + ":8080/jaxws-jbws2206/MetroTestServiceBean?wsdl");
      Service service = Service.create(wsdlURL, new QName("http://com.acme.metroexceptiontest/", "MetroTestService"));
      MetroTestPortType port = service.getPort(MetroTestPortType.class);

      String res = port.helloWorld();
      assertEquals("HelloWorld", res);
   }

   public void testImportedDocument() throws Exception
   {
      URL wsdlURL = new URL("http://" + getServerHost() + ":8080/jaxws-jbws2206/MetroTestServiceBean?wsdl");
      WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
      Definition wsdlDefinition = wsdlReader.readWSDL(wsdlURL.toString());
      assertNotNull(wsdlDefinition);
      Types types = wsdlDefinition.getTypes();
      Map<String, List<SchemaImport>> map = null;
      for (ExtensibilityElement el : (List<ExtensibilityElement>)types.getExtensibilityElements())
      {
         if (el instanceof Schema)
         {
            map = (Map<String, List<SchemaImport>>)((Schema)el).getImports();
         }
      }
      assertNotNull(map);
      SchemaImport schemaImport = map.get("http://com.acme.metroexceptiontest/").iterator().next();
      assertNotNull(schemaImport);
      String schemaLocation = schemaImport.getSchemaLocationURI();
      assertNotNull(schemaLocation);
      assertTrue(schemaLocation.startsWith("http://" + getServerHost())); //check absolute url
      
      
      HttpURLConnection con = (HttpURLConnection)new URL(schemaLocation).openConnection();
      BufferedReader isr = new BufferedReader(new InputStreamReader(con.getInputStream()));
      String s = isr.readLine();
      StringBuilder sb = new StringBuilder();
      while (s != null)
      {
         sb.append(s);
         sb.append("\n");
         s = isr.readLine();
      }
      assertTrue(sb.toString().contains("MetroTestFaultInfo"));
   }
}
