package org.jboss.test.ws.jaxws.jbws2308;

import javax.xml.namespace.QName;
import javax.xml.soap.Detail;
import javax.xml.soap.DetailEntry;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPFactory;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.ws.handler.LogicalHandler;
import javax.xml.ws.handler.LogicalMessageContext;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.soap.SOAPFaultException;

import org.jboss.logging.Logger;

public class ClientHandler3 implements LogicalHandler<LogicalMessageContext>
{
   private static Logger log = Logger.getLogger(ClientHandler3.class);
   private final QName FAULTCODE = new QName("http://dlhandlerservice.org/wsdl", "ItsASoapFault", "tns");
   private static MessageFactory mfactorySOAP11;
   private static MessageFactory mfactorySOAP12;
   
   public void close(MessageContext arg0)
   {
      
   }

   public boolean handleFault(LogicalMessageContext arg0)
   {
      System.out.println("HANDLE FAULT 3");
      return true;
   }

   public boolean handleMessage(LogicalMessageContext arg0)
   {
      System.out.println("HANDLE MESSAGE 3");
      String faultString = "ClientLogicalHandler6.handleMessage throws SOAPFaultException for ClientLogicalOutboundHandleFaultFalseTest";
      SOAPFault sf = null;
      try
      {
         Name name = SOAPFactory.newInstance().createName("somefaultentry");
         sf = createSOAPFault("soap11", this.FAULTCODE, "faultActor", faultString, name);
      }
      catch (Exception e)
      {
         throw new RuntimeException("Unexpected error occurred in handleMessage for an outbound message" + e);
      }
      throw new SOAPFaultException(sf);
   }

   private static SOAPFault createSOAPFault(String soapVer, QName faultCode, String faultActor, String faultString, Name name) throws Exception
   {
      DetailEntry detailentry;
      SOAPFault soapFault = null;
      try
      {
         log.info("Create a SOAP MessageFactory instance - " + soapVer);
         MessageFactory mfactory = getMessageFactory(soapVer);

         log.info("Create a SOAPMessage");
         SOAPMessage soapmsg = mfactory.createMessage();

         log.info("Get SOAP Part");
         SOAPPart sp = soapmsg.getSOAPPart();

         log.info("Get SOAP Envelope");
         SOAPEnvelope envelope = sp.getEnvelope();

         log.info("Get SOAP Header");
         SOAPHeader hdr = envelope.getHeader();

         log.info("Get SOAP Body");
         SOAPBody body = envelope.getBody();

         soapFault = body.addFault();
         soapFault.setFaultCode(faultCode);
         soapFault.setFaultActor(faultActor);
         soapFault.setFaultString(faultString);
         Detail detail = soapFault.addDetail();
         detailentry = detail.addDetailEntry(name);
      }
      catch (Exception e)
      {
         log.info("Exception caught: " + e);
         return null;
      }
      return soapFault;
   }

   public static MessageFactory getMessageFactory(String soapVer) throws Exception
   {
      System.out.println("JAXWS_Util:getMessageFactory");
      if (soapVer.equals("soap11"))
      {
         if (mfactorySOAP11 == null)
            mfactorySOAP11 = MessageFactory.newInstance();
         return mfactorySOAP11;
      }
      if (mfactorySOAP12 == null)
         mfactorySOAP12 = MessageFactory.newInstance("SOAP 1.2 Protocol");
      return mfactorySOAP12;
   }
   
}
