/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.jbws2403;

import junit.framework.Assert;
import junit.framework.Test;
import org.jboss.wsf.test.JBossWSTest;
import org.jboss.wsf.test.JBossWSTestSetup;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.net.URL;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.ws.Holder;
import javax.xml.ws.Service;

/**
 * Test client 
 * @author richard.opalka@jboss.com
 */
public class JBWS2403TestCase extends JBossWSTest
{
   private TestPort port;
   private URL url;

   public static Test suite()
   {
      return new JBossWSTestSetup(JBWS2403TestCase.class, "jaxws-jbws2403.war");
   }

   protected void setUp() throws Exception
   {
      super.setUp();
      URL wsdlURL = new URL("http://" + getServerHost() + ":8080/jaxws-jbws2403/Endpoint?wsdl");
      Service service = Service.create(wsdlURL, new QName("http://TestService.org/wsdl", "TestService"));
      port = service.getPort(TestPort.class);
      url = new URL("http://" + getServerHost() + ":8080/jaxws-jbws2403/file.xml");
   }

   protected void tearDown() throws Exception
   {
      port = null;
      url = null;
      super.tearDown();
   }

   public void test() throws Exception
   {
      Request request = new Request();
      request.setMimeType("text/xml");
      request.setUrl(url.toString());
      Holder<Source> attach = new Holder<Source>();
      Holder<Response> response = new Holder<Response>();
      port.echo(request, response, attach);
      if(!validateAttachment(request, response.value, attach))
         Assert.fail("Attachments differ");
   }

   private boolean validateAttachment(Request request, Response response, Holder<Source> attach) throws Exception
   {
      boolean result=true;

      System.out.println("Check if the attachment mime types are correct");
      if(!response.getMimeType().equals(request.getMimeType()))
      {
         System.err.println("MimeType is not equal in request and response");
         System.err.println("Request MimeType = " + request.getMimeType());
         System.err.println("Response MimeType = " + response.getMimeType());
         result=false;
      }

      System.out.println("Check if the attachment contents are correct");
      DataHandler dh1 = new DataHandler(url);
      byte data1[] = new byte[4096];
      byte data2[] = new byte[4096];
      dh1.getInputStream().read(data1, 0, 4096);
      ((StreamSource)attach.value).getInputStream().read(data2, 0, 4096);
      if(!validateAttachmentData(new String(data1), new String(data2)))
         result=false;

      return result;
   }

   private boolean validateAttachmentData(String data1, String data2) 
   {
      // It's hard to compare XML messages that are XML compatible but not identical (see JBWS-2403 comments)
      // Because of this issue we're testing for specific strings if they are present in both attachments.
      assertTrue(data1.indexOf("Header1") != -1);
      assertTrue(data1.indexOf("Header2") != -1);
      assertTrue(data1.indexOf("Header3") != -1);
      assertTrue(data1.indexOf("Child1") != -1);
      assertTrue(data1.indexOf("Child2") != -1);
      assertTrue(data1.indexOf("Child3") != -1);
      assertTrue(data1.indexOf("</env:Envelope>") != -1);
      
      assertTrue(data2.indexOf("Header1") != -1);
      assertTrue(data2.indexOf("Header2") != -1);
      assertTrue(data2.indexOf("Header3") != -1);
      assertTrue(data2.indexOf("Child1") != -1);
      assertTrue(data2.indexOf("Child2") != -1);
      assertTrue(data2.indexOf("Child3") != -1);
      assertTrue(data2.indexOf("</env:Envelope>") != -1);

      return true;
   }
}
